==> libremail-2.4.tar.bz2: libremail/src/modepage.c <==
/*
    Fichier modepage.c
    Auteur Bernard Chardonneau

    Logiciel libre, droits d'utilisation prciss en franais
    dans le fichier : licence-fr.txt

    Traductions des droits d'utilisation dans les fichiers :
    licence-de.txt , licence-en.txt , licence-es.txt ,
    licence-it.txt , licence-nl.txt , licence-pt.txt ,
    licence-eo.txt , licence-eo-utf.txt

    Droits d'utilisation galement sur la page web :
    http://libremail.tuxfamily.org/voir.php?page=droits


    Bibliothque de fonctions permettant de grer l'affichage en mode page

    Certaines de ces fonctions sont des macros de modepage.h

    On peut remarquer que les fonctions de dplacement dans le texte
    et d'affichage d'une page ne manipulent que des indicateurs
    de position dans le texte.

    La fonction affligne reste dans le fichier source principal de
    chaque application, ce qui permet d'avoir un affichage adapt
    aux donnes  traiter.
*/

#define modepage  // pour la dclaration de variables globales  l'application

#include <stdio.h>
#include <stdlib.h>
#include "modepage.h"
#include "carspe.h"



/* affichage en surbrillance */
void clair()
{
    static char sequence [8] = ""; // squence de changement de couleur
    char *valenv;   // contenu de la variable d'environnement libremail_bright
    int  valcoul;   // couleur pour la squence d'chappement


    // si aucune squence d'chappement dja mmorise
    if (! *sequence)
    {
        // tenter de rcuprer la variable d'environnement libremail_bright
        valenv = getenv ("libremail_bright");

        // si variable d'environnement utilise
        if (valenv)
        {
            // convertir le contenu de la variable en un numro de couleur
            valcoul = (valenv [0] & 1) + ((valenv [1] & 1) * 2)
                                       + ((valenv [2] & 1) * 4);

            // et gnrer la squence d'chappement avec surbrillance
            sprintf (sequence, "%c[%d;1m", ESC, valcoul + 30);
        }
        else
            // sinon gnrer la squence d'chappement pour passer surbrillance
            sprintf (sequence, "%c[1m", ESC);
    }

    // envoyer la squence d'chappement
    printf (sequence);
}



/* affichage en bleu fonc par dfaut */
void sombre()
{
    static char sequence [6] = ""; // squence de changement de couleur
    char *valenv;   // contenu de la variable d'environnement libremail_dark
    int  valcoul;   // couleur pour la squence d'chappement


    // si aucune squence d'chappement dja mmorise
    if (! *sequence)
    {
        // tenter de rcuprer la variable d'environnement libremail_dark
        valenv = getenv ("libremail_dark");

        // si variable d'environnement utilise
        if (valenv)
        {
            // convertir le contenu de la variable en un numro de couleur
            valcoul = (valenv [0] & 1) + ((valenv [1] & 1) * 2)
                                       + ((valenv [2] & 1) * 4);

            // et gnrer la squence d'chappement
            sprintf (sequence, "%c[%dm", ESC, valcoul + 30);
        }
        else
            // sinon gnrer la squence d'chappement pour passer en bleu fonc
            sprintf (sequence, "%c[34m", ESC);
    }

    // envoyer la squence d'chappement
    printf (sequence);
}



/* raffiche la page d'cran courante en tenant
   compte du changement de taille de la fentre */

void affpage ()
{
    int i, j;


    // rcuprer la taille de la fentre (qui peut avoir chang)
    lig_col ();

    // se positionner en haut d'cran et l'effacer
    debpage ();
    effpage ();

    // initialisation
    i = 1;
    j = lignecour - lignecran + 1;

    // rafficher toutes les lignes de la page
    while (i++ < lignepage && j < nb_lignes)
    {
        affligne (j++);
        putchar ('\n');
    }

    // remonter  la ligne sur laquelle on tait
    while (--i > lignecran)
        montecurs ();
}



/* remonte de n lignes (max) dans le texte affich */

void monte (int n)
{
    // limiter le dplacement pour ne pas sortir du texte
    if (n > lignecour)
        n = lignecour;

    // tant qu'on n'est pas en haut de page et dplacement non termin
    while (n > 0 && lignecran > 1)
    {
        // remonter d'une ligne
        montecurs ();

        // et mettre  jour les variables
        lignecran --;
        lignecour --;
        n--;
    }

    // si dplacement de plus d'une page
    if (n > lignepage)
    {
        // on n'affichera pas les lignes intermdiaires
        lignecour = lignecour + lignepage - n;
        n = lignepage;
    }

    // tant que dplacement non termin
    while (n-- > 0)
    {
        // descendre le texte affich d'une ligne
        insligne ();

        // et afficher la ligne  rajouter en haut d'cran
        affligne (--lignecour);
        putchar ('\r');
    }
}



/* descend de n lignes (max) dans le texte affich */

void descend (int n)
{
    // limiter le dplacement pour ne pas sortir du texte
    if (n > nb_lignes - lignecour - 1)
        n = nb_lignes - lignecour - 1;

    // tant qu'on n'est pas en bas de page et dplacement non termin
    while (n > 0 && lignecran < lignepage - 1)
    {
        // descendre d'une ligne
        putchar ('\n');

        // et mettre  jour les variables
        lignecran ++;
        lignecour ++;
        n--;
    }

    // si dplacement de plus d'une page
    if (n > lignepage)
    {
        // on n'affichera pas les lignes intermdiaires
        lignecour = lignecour + n - lignepage;
        n = lignepage;
    }

    // tant que dplacement non termin
    while (n-- > 0)
    {
        // afficher la ligne  rajouter en bas de page
        // et remonter tout le texte d'une ligne
        putchar ('\n');
        affligne (++lignecour);
        putchar ('\n');

        // remonter sur la ligne insre
        montecurs ();
    }
}



/* rcupre le nombre de lignes et de colonnes d'affichage */

void lig_col ()
{
    char commande [28];
    FILE *fcaract;


    // rcuprer les caractristiques du terminal
    sprintf (commande, "stty size > /tmp/fich%X", getpid ());
    system (commande);

    // ouvrir le fichier cr par la commande  stty
    fcaract = fopen (commande + 12, "r");

    // si ouverture russie
    if (fcaract)
    {
        // lire le nombre de lignes et de colonnes
        fscanf (fcaract, "%d%d", &lignepage, &colonpage);

        // fermer et dtruire le fichier
        fclose (fcaract);
        unlink (commande + 12);
    }
    // cas trs particulier : problme de droits d'accs  /tmp
    else
    {
        // message d'avertissement
     // "Droit d'accs insuffisants  /tmp, le fonctionnement peut tre dgrad"
        affiche_msg ("PB_ACCES_TMP");
        sleep (3);

        // on prend les valeurs par dfaut
        lignepage = defaut_ligne;
        colonpage = defaut_colon;
    }

    // corriger avec des valeurs par dfaut si l'on a rcupr des 0
    // (par exemple depuis une fentre telnet)
    if (! lignepage)
        lignepage = defaut_ligne;

    if (! colonpage)
        colonpage = defaut_colon;
}



/* lit un caractre au clavier et le traite si caractre spcial */

int leccar ()
{
    int car, carsuiv;

    // attendre qu'un caractre soit tap
    do
        car = getchar ();
    while (car == -1);

    // si caractre ordinaire, on le retourne
    if (car != ESC)
        return (car);

    // lire le caractre qui suit le ESC
    car = getchar ();

    // traitement du caractre ESC et des squences d'chappement
    if (car == '[' || car == 'O')
    {
        car = getchar ();

        if ('1' <= car && car <= '9')
        {
            // squence  ESC[n~  o  n  est une valeur numrique
            carsuiv = getchar ();
            car = car & 0x0F;

            // rcuprer la valeur numrique
            while ('0' <= carsuiv && carsuiv <= '9')
            {
                car = (car * 10) + (carsuiv & 0x0F);
                carsuiv = getchar ();
            }

            if (carsuiv == '~')
                return (CARSPE | car);
            else
                // "Caractre spcial non trait"
                affiche_msg ("CARSPE_INVALIDE");
        }
        else if (car == '[')
        {
            // squence  ESC[[n  (certaines touches de fonction)
            car = getchar ();
            return (CARSPE + 0x0F + car);
        }
        else
            // squence  ESC[lettre
            return (CARSPE | car);
    }

    // caractre ESC classique
    else
    {
        // si aprs le ESC on a eu un autre caractre de suite
        // le remettre dans le buffer
        if (car != -1)
            ungetc (car, stdin);

        // retour du caractre ESCape
        return (ESC);
    }
}



/* excute une commande, puis repasse en mode page */

void execom (char *outil, char *fichier)
{
    char commande [130];    // chaine contenant la commande  excuter
    unsigned long heuredeb; // pour dtecter problme excution commandes


    // effacer le bas de page
    effpage ();
    fflush (stdout);

    // fabriquer la commande  xcuter
    sprintf (commande, "%s %s", outil, fichier);

    // rcuprer l'heure courante
    heuredeb = time (0);

    // lancer la commande
    system (commande);

    // si la commande a rendu la main trop rapidement
    // et que c'est une commande interactive
    if ((time (0) <= heuredeb + 1) && (strcmp (outil, "lp") != 0)
                                   && (strcmp (outil, "recuppj") != 0))
    {
        // attendre lecture du message d'erreur
        sleep (2);
    }

    // reconfigurer la voie que commande a remit en mode normal en se terminant
    mode_raw ();
}

==> libremail-2.5.2.tar.bz2: libremail/src/modepage.c <==
/*
    Fichier modepage.c
    Auteur Bernard Chardonneau

    Logiciel libre, droits d'utilisation prciss en franais
    dans le fichier : licence-fr.txt

    Traductions des droits d'utilisation dans les fichiers :
    licence-de.txt , licence-en.txt , licence-es.txt ,
    licence-it.txt , licence-nl.txt , licence-pt.txt ,
    licence-eo.txt , licence-eo-utf.txt

    Droits d'utilisation galement sur la page web :
    http://libremail.tuxfamily.org/voir.php?page=droits


    Bibliothque de fonctions permettant de grer l'affichage en mode page

    Certaines de ces fonctions sont des macros de modepage.h

    On peut remarquer que les fonctions de dplacement dans le texte
    et d'affichage d'une page ne manipulent que des indicateurs
    de position dans le texte.

    La fonction affligne reste dans le fichier source principal de
    chaque application, ce qui permet d'avoir un affichage adapt
    aux donnes  traiter.
*/

#define modepage  // pour la dclaration de variables globales  l'application

#include <stdio.h>
#include <stdlib.h>
#include "modepage.h"
#include "carspe.h"



/* affichage en surbrillance */
void clair()
{
    static char sequence [8] = ""; // squence de changement de couleur
    char *valenv;   // contenu de la variable d'environnement libremail_bright
    int  valcoul;   // couleur pour la squence d'chappement


    // si aucune squence d'chappement dja mmorise
    if (! *sequence)
    {
        // tenter de rcuprer la variable d'environnement libremail_bright
        valenv = getenv ("libremail_bright");

        // si la variable d'environnement existe
        if (valenv)
        {
            // convertir le contenu de la variable en un numro de couleur
            valcoul = (valenv [0] & 1) + ((valenv [1] & 1) * 2)
                                       + ((valenv [2] & 1) * 4);

            // et gnrer la squence d'chappement avec couleur en surbrillance
            sprintf (sequence, "%c[%d;1m", ESC, valcoul + 30);
        }
        else
            // sinon gnrer la squence d'chappement pour passer surbrillance
            sprintf (sequence, "%c[1m", ESC);
    }

    // envoyer la squence d'chappement
    printf ("%s", sequence);
}



/* affichage en bleu fonc par dfaut */
void sombre()
{
    static char sequence [6] = ""; // squence de changement de couleur
    char *valenv;   // contenu de la variable d'environnement libremail_dark
    int  valcoul;   // couleur pour la squence d'chappement


    // si aucune squence d'chappement dja mmorise
    if (! *sequence)
    {
        // tenter de rcuprer la variable d'environnement libremail_dark
        valenv = getenv ("libremail_dark");

        // si variable d'environnement utilise
        if (valenv)
        {
            // convertir le contenu de la variable en un numro de couleur
            valcoul = (valenv [0] & 1) + ((valenv [1] & 1) * 2)
                                       + ((valenv [2] & 1) * 4);

            // et gnrer la squence d'chappement
            sprintf (sequence, "%c[%dm", ESC, valcoul + 30);
        }
        else
            // sinon gnrer la squence d'chappement pour passer en bleu fonc
            sprintf (sequence, "%c[34m", ESC);
    }

    // envoyer la squence d'chappement
    printf ("%s", sequence);
}



/* raffiche la page d'cran courante en tenant
   compte du changement de taille de la fentre */

void affpage ()
{
    int i, j;


    // rcuprer la taille de la fentre (qui peut avoir chang)
    lig_col ();

    // se positionner en haut d'cran et l'effacer
    debpage ();
    effpage ();

    // initialisation
    i = 1;
    j = lignecour - lignecran + 1;

    // rafficher toutes les lignes de la page
    while (i++ < lignepage && j < nb_lignes)
    {
        affligne (j++);
        putchar ('\n');
    }

    // remonter  la ligne sur laquelle on tait
    while (--i > lignecran)
        montecurs ();
}



/* remonte de n lignes (max) dans le texte affich */

void monte (int n)
{
    // limiter le dplacement pour ne pas sortir du texte
    if (n > lignecour)
        n = lignecour;

    // tant qu'on n'est pas en haut de page et dplacement non termin
    while (n > 0 && lignecran > 1)
    {
        // remonter d'une ligne
        montecurs ();

        // et mettre  jour les variables
        lignecran --;
        lignecour --;
        n--;
    }

    // si dplacement de plus d'une page
    if (n > lignepage)
    {
        // on n'affichera pas les lignes intermdiaires
        lignecour = lignecour + lignepage - n;
        n = lignepage;
    }

    // tant que dplacement non termin
    while (n-- > 0)
    {
        // descendre le texte affich d'une ligne
        insligne ();

        // et afficher la ligne  rajouter en haut d'cran
        affligne (--lignecour);
        putchar ('\r');
    }
}



/* descend de n lignes (max) dans le texte affich */

void descend (int n)
{
    // limiter le dplacement pour ne pas sortir du texte
    if (n > nb_lignes - lignecour - 1)
        n = nb_lignes - lignecour - 1;

    // tant qu'on n'est pas en bas de page et dplacement non termin
    while (n > 0 && lignecran < lignepage - 1)
    {
        // descendre d'une ligne
        putchar ('\n');

        // et mettre  jour les variables
        lignecran ++;
        lignecour ++;
        n--;
    }

    // si dplacement de plus d'une page
    if (n > lignepage)
    {
        // on n'affichera pas les lignes intermdiaires
        lignecour = lignecour + n - lignepage;
        n = lignepage;
    }

    // tant que dplacement non termin
    while (n-- > 0)
    {
        // afficher la ligne  rajouter en bas de page
        // et remonter tout le texte d'une ligne
        putchar ('\n');
        affligne (++lignecour);
        putchar ('\n');

        // remonter sur la ligne insre
        montecurs ();
    }
}



/* rcupre le nombre de lignes et de colonnes d'affichage */

void lig_col ()
{
    char commande [28];
    FILE *fcaract;


    // rcuprer les caractristiques du terminal
    sprintf (commande, "stty size > /tmp/fich%X", getpid ());
    system (commande);

    // ouvrir le fichier cr par la commande  stty
    fcaract = fopen (commande + 12, "r");

    // si ouverture russie
    if (fcaract)
    {
        // lire le nombre de lignes et de colonnes
        fscanf (fcaract, "%d%d", &lignepage, &colonpage);

        // fermer et dtruire le fichier
        fclose (fcaract);
        unlink (commande + 12);
    }
    // cas trs particulier : problme de droits d'accs  /tmp
    else
    {
        // message d'avertissement
     // "Droit d'accs insuffisants  /tmp, le fonctionnement peut tre dgrad"
        affiche_msg_nocr ("PB_ACCES_TMP");
        attendre (3);

        // on prend les valeurs par dfaut
        lignepage = defaut_ligne;
        colonpage = defaut_colon;
    }

    // corriger avec des valeurs par dfaut si l'on a rcupr des 0
    // (par exemple depuis une fentre telnet)
    if (! lignepage)
        lignepage = defaut_ligne;

    if (! colonpage)
        colonpage = defaut_colon;
}



/* lit un caractre au clavier et le traite si caractre spcial */

int leccar ()
{
    int car, carsuiv;

    // attendre qu'un caractre soit tap
    do
        car = getchar ();
    while (car == -1);

    // si caractre ordinaire, on le retourne
    if (car != ESC)
        return (car);

    // lire le caractre qui suit le ESC
    car = getchar ();

    // traitement du caractre ESC et des squences d'chappement
    if (car == '[' || car == 'O')
    {
        car = getchar ();

        if ('1' <= car && car <= '9')
        {
            // squence  ESC[n~  o  n  est une valeur numrique
            carsuiv = getchar ();
            car = car & 0x0F;

            // rcuprer la valeur numrique
            while ('0' <= carsuiv && carsuiv <= '9')
            {
                car = (car * 10) + (carsuiv & 0x0F);
                carsuiv = getchar ();
            }

            if (carsuiv == '~')
                return (CARSPE | car);
            else
                // "Caractre spcial non trait"
                affiche_msg ("CARSPE_INVALIDE");
        }
        else if (car == '[')
        {
            // squence  ESC[[n  (certaines touches de fonction)
            car = getchar ();
            return (CARSPE + 0x0F + car);
        }
        else
            // squence  ESC[lettre
            return (CARSPE | car);
    }

    // caractre ESC classique
    else
    {
        // si aprs le ESC on a eu un autre caractre de suite
        // le remettre dans le buffer
        if (car != -1)
            ungetc (car, stdin);

        // retour du caractre ESCape
        return (ESC);
    }
}



/* excute une commande, puis repasse en mode page */

void execom (char *outil, char *fichier)
{
    char commande [130];    // chaine contenant la commande  excuter
    unsigned long heuredeb; // pour dtecter problme excution commandes


    // effacer le bas de page
    effpage ();
    fflush (stdout);

    // fabriquer la commande  xcuter
    sprintf (commande, "%s %s", outil, fichier);

    // rcuprer l'heure courante
    heuredeb = time (0);

    // lancer la commande
    system (commande);

    // si la commande a rendu la main trop rapidement
    // et que c'est une commande interactive
    if ((time (0) <= heuredeb + 1) && (strcmp (outil, "lp") != 0)
                                   && (strcmp (outil, "recuppj") != 0))
    {
        // attendre lecture du message d'erreur
        montecurs ();
        attendre (2);
    }

    // reconfigurer la voie que commande a remit en mode normal en se terminant
    mode_raw ();
}



/* attente en seconde pour l'affichage d'un message qui sera ensuite effac */

void attendre (int duree)
{
    char *varenv;   // contenu variable d'environnement libremail_errtimeout


    // si la variable d'environnement libremail_errtimeout est utilise
    varenv = getenv ("libremail_errtimeout");

    if (varenv)
    {
        // rcuprer la dure de l'attente qu'elle contient
        duree = atoi (varenv);

        // positionner le curseur en dbut de ligne
        putchar ('\r');
    }

    // si attente pour une dure limite
    if (duree)
    {
        // forcer l'affichage du texte en attente
        fflush (stdout);

        // attendre pour la dure demande
        sleep (duree);
    }
    // sinon attendre que l'utilisateur tape un caractre
    else
        leccar ();
}
